@@ -210,6 +210,18 @@ class InstanceDatastore(NamedTuple):
     Id: int
     FriendlyName: str
 
+class InstanceStatus(NamedTuple):
+    """
+    An instance status object -- see ADSModule#GetInstanceStatuses()
+    Author: p0t4t0sandwich
+    :param InstanceID: The instance ID
+    :type InstanceID: UUID
+    :param Running: Whether the instance is running
+    :type Running: bool
+    """
+    InstanceID: UUID
+    Running: bool
+
 class State(Enum):
     """
     Represents the state of an instance
@@ -385,7 +397,7 @@ class Instance():
     DeploymentArgs: dict[str, str]
     DisplayImageSource: str
 
-    def __init__(self, InstanceID: UUID, TargetID: UUID, InstanceName: str, FriendlyName: str, Module: str, AMPVersion: AMPVersionAlias, IsHTTPS: bool, IP: str, Port: int, Daemon: bool, DaemonAutostart: bool, ExcludeFromFirewall: bool, Running: bool, AppState: State, Tags: list[str], DiskUsageMB: int, ReleaseStream: str, ManagementMode: str, Suspended: bool, IsContainerInstance: bool, ContainerMemoryMB: int, ContainerMemoryPolicy: str, ContainerCPUs: int, Metrics: dict[str, Metric], ApplicationEndpoints: list[EndpointInfo], DeploymentArgs: dict[str, str], DisplayImageSource: str = "", Description: str = "", ModuleDisplayName: str = "", SpecificDockerImage: str = "") -> None:
+    def __init__(self, InstanceID: UUID, TargetID: UUID, InstanceName: str, FriendlyName: str, Module: str, AMPVersion: AMPVersionAlias, IsHTTPS: bool, IP: str, Port: int, Daemon: bool, DaemonAutostart: bool, ExcludeFromFirewall: bool, Running: bool, AppState: State, Tags: list[str], DiskUsageMB: int, ReleaseStream: str, ManagementMode: str, Suspended: bool, IsContainerInstance: bool, ContainerMemoryMB: int, ContainerMemoryPolicy: str, ContainerCPUs: int, ApplicationEndpoints: list[EndpointInfo], DeploymentArgs: dict[str, str], Metrics: dict[str, Metric] = {}, DisplayImageSource: str = "", Description: str = "", ModuleDisplayName: str = "", SpecificDockerImage: str = "") -> None:
         """
         Initializes the Instance object
         Author: p0t4t0sandwich
@@ -420,10 +432,42 @@ class Instance():
         self.DeploymentArgs = DeploymentArgs
         self.DisplayImageSource = DisplayImageSource
 
+class GlibcInfo():
+    """
+    Glibc information object
+    Author: Kei
+    """
+    Major : int
+    Minor : int
+    Build : int
+    Revision : int
+    MajorRevision : int
+    MinorRevision : int
+
+    def __init__(self, Major: int, Minor: int, Build: int, Revision: int, MajorRevision: int, MinorRevision: int) -> None:
+        """
+        Initializes the GlibcInfo object
+        Author: Kei
+        """
+        self.Major = Major
+        self.Minor = Minor
+        self.Build = Build
+        self.Revision = Revision
+        self.MajorRevision = MajorRevision
+        self.MinorRevision = MinorRevision
+
+# Type alias for State
+# Author: Kei
+GlibcInfoAlias = GlibcInfo
+
 class PlatformInfo():
     """
     Platform information object
     Author: p0t4t0sandwich
+    :param IsSharedSetup: If the setup is shared
+    :type IsSharedSetup: bool
+    :param AdminRights: Admin right status
+    :type AdminRights: int
     :param CPUInfo: The CPU information object
     :type CPUInfo: CPUInfo
     :param InstalledRAMMB: The installed RAM in MB
@@ -432,34 +476,142 @@ class PlatformInfo():
     :type OS: int
     :param PlatformName: The platform name
     :type PlatformName: str
+    :param HardwarePlatformName: The hardware platform name
+    :type HardwarePlatformName: str
     :param SystemType: The system type
     :type SystemType: int
     :param Virtualization: The virtualization
     :type Virtualization: int
+    :param InstalledGlibcVersion: Version of Glibc installed
+    :type InstalledGlibcVersion: GlibcInfo
+    :param MaxSwapMB: The maximum swap in MB
+    :type MaxSwapMB: int
+    :param ContainerSwapMB: The container swap in MB
+    :type ContainerSwapMB: int
     """
+    IsSharedSetup: bool
+    AdminRights: int
     CPUInfo: CPUInfoAlias
     InstalledRAMMB: int
     OS: int
     PlatformName: str
+    HardwarePlatformName: str
     SystemType: int
     Virtualization: int
+    InstalledGlibcVersion: GlibcInfo
+    MaxSwapMB: int
+    ContainerSwapMB: int
 
-    def __init__(self, CPUInfo: CPUInfoAlias, InstalledRAMMB: int, OS: int, PlatformName: str, SystemType: int, Virtualization: int) -> None:
+    def __init__(self, CPUInfo: CPUInfoAlias, InstalledRAMMB: int, OS: int, PlatformName: str, SystemType: int, Virtualization: int, InstalledGlibcVersion: GlibcInfoAlias = None, IsSharedSetup: bool = False, AdminRights: int = -1, HardwarePlatformName: str = "", MaxSwapMB: int = 0, ContainerSwapMB: int = 0) -> None:
         """
         Initializes the PlatformInfo object
         Author: p0t4t0sandwich
         """
+        self.IsSharedSetup = IsSharedSetup
+        self.AdminRights = AdminRights
         self.CPUInfo = CPUInfoAlias(**CPUInfo)
         self.InstalledRAMMB = InstalledRAMMB
         self.OS = OS
         self.PlatformName = PlatformName
+        self.HardwarePlatformName = HardwarePlatformName
         self.SystemType = SystemType
         self.Virtualization = Virtualization
+        if InstalledGlibcVersion != None:
+            self.InstalledGlibcVersion = GlibcInfoAlias(**InstalledGlibcVersion)
+        self.MaxSwapMB = MaxSwapMB
+        self.ContainerSwapMB = ContainerSwapMB
 
 # Type alias for PlatformInfo
 # Author: p0t4t0sandwich
 PlatformInfoAlias = PlatformInfo
 
+class FileDirectory(NamedTuple):
+    """
+    A file directory object -- see FileManagerPlugin#GetDirectoryListing(str)
+    Author: p0t4t0sandwich
+    :param IsDirectory: Whether the file is a directory
+    :type IsDirectory: bool
+    :param IsVirtualDirectory: Whether the file is a virtual directory
+    :type IsVirtualDirectory: bool
+    :param Filename: The filename
+    :type Filename: str
+    :param SizeBytes: The size in bytes
+    :type SizeBytes: float
+    :param Created: The creation date
+    :type Created: str
+    :param Modified: The modification date
+    :type Modified: str
+    :param IsDownloadable: Whether the file is downloadable
+    :type IsDownloadable: bool
+    :param IsEditable: Whether the file is editable
+    :type IsEditable: bool
+    :param IsArchive: Whether the file is an archive
+    :type IsArchive: bool
+    :param IsExcludedFromBackups: Whether the file is excluded from backups
+    :type IsExcludedFromBackups: bool
+    """
+    IsDirectory: bool
+    IsVirtualDirectory: bool
+    Filename: str
+    SizeBytes: float
+    Created: str
+    Modified: str
+    IsDownloadable: bool
+    IsEditable: bool
+    IsArchive: bool
+    IsExcludedFromBackups: bool
+
+class FitnessInfo():
+    """
+    Fitness information object
+    Author: Kei
+    :param Available: Availability
+    :type Available: bool
+    :param TotalServices: Service count.
+    :type TotalServices: int
+    :param FreeRAMMB: Unallocated RAM in MB.
+    :type FreeRAMMB: int
+    :param FreeDiskMB: Unallocated disk space in MB.
+    :type FreeDiskMB: int
+    :param CPUServiceRatio: CPU service ratio.
+    :type CPUServiceRatio: float
+    :param ThreadQueueLength: Thread queue length.
+    :type ThreadQueueLength: int
+    :param LoadAvg: Load average.
+    :type LoadAvg: float
+    :param ReminaingInstanceSlots: Remaining instance slots.
+    :type ReminaingInstanceSlots: int
+    :param Score: Fitness score.
+    :type Score: float
+    """
+    Available: bool
+    TotalServices: int
+    FreeRAMMB: int
+    FreeDiskMB: int
+    CPUServiceRatio: float
+    ThreadQueueLength: int
+    LoadAvg: float
+    ReminaingInstanceSlots: int
+    Score: float
+
+    def __init__(self, Available: bool, TotalServices: int, FreeRAMMB: int, FreeDiskMB: int, CPUServiceRatio: float, ThreadQueueLength: int, LoadAvg: float, ReminaingInstanceSlots: int, Score: float) -> None:
+        """
+        Initializes the FitnessInfo object
+        """
+        self.Available = Available
+        self.TotalServices = TotalServices
+        self.FreeRAMMB = FreeRAMMB
+        self.FreeDiskMB = FreeDiskMB
+        self.CPUServiceRatio = CPUServiceRatio
+        self.ThreadQueueLength = ThreadQueueLength
+        self.LoadAvg = LoadAvg
+        self.ReminaingInstanceSlots = ReminaingInstanceSlots
+        self.Score = Score
+
+# Type alias for FitnessInfo
+# Author: Kei        
+FitnessInfoAlias = FitnessInfo
+
 class IADSInstance():
     """
     An ADS instance object
@@ -474,6 +626,8 @@ class IADSInstance():
     :type Description: str
     :param Disabled: Whether the instance is disabled
     :type Disabled: bool
+    :param Fitness: Fitness of the instance.
+    :type Fitness: FitnessInfo
     :param IsRemote: Whether the instance is remote
     :type IsRemote: bool
     :param Platform: The platform information object
@@ -500,11 +654,12 @@ class IADSInstance():
     FriendlyName: str
     Description: str
     Disabled: bool
+    Fitness: FitnessInfo
     IsRemote: bool
     Platform: PlatformInfo
     Datastores: list[InstanceDatastore]
     CreatesInContainers: bool
-    State: State
+    State: StateAlias
     StateReason: str
     CanCreate: bool
     LastUpdated: str
@@ -514,7 +669,7 @@ class IADSInstance():
     TagsList: list[str]
     URL: str
 
-    def __init__(self, Id: int, InstanceId: UUID, FriendlyName: str, Disabled: bool, IsRemote: bool, Platform: PlatformInfo, Datastores: list[InstanceDatastore], CreatesInContainers: bool, State: StateAlias, StateReason: str, CanCreate: bool, LastUpdated: str, AvailableInstances: list[Instance], AvailableIPs: list[str], Description: str = "", Tags: list[str] = [], TagsList: list[str] = [], URL: str = "") -> None:
+    def __init__(self, Id: int, InstanceId: UUID, FriendlyName: str, Disabled: bool, IsRemote: bool, Datastores: list[InstanceDatastore], CreatesInContainers: bool, State: StateAlias, CanCreate: bool, LastUpdated: str, AvailableInstances: list[Instance], AvailableIPs: list[str], Platform: PlatformInfo = None, Description: str = "", Tags: list[str] = [], TagsList: list[str] = [], URL: str = "", StateReason: str = "", Fitness: FitnessInfo = None) -> None:
         """
         Initializes the IADSInstance object
         Author: p0t4t0sandwich
@@ -524,18 +679,19 @@ class IADSInstance():
         self.FriendlyName = FriendlyName
         self.Description = Description
         self.Disabled = Disabled
+        if Fitness != None:
+            self.Fitness = Fitness
         self.IsRemote = IsRemote
-        self.Platform = PlatformInfo(**Platform)
+        if Platform != None:
+            self.Platform = PlatformInfo(**Platform)
+        else:
+            self.Platform = Platform
         self.Datastores = [InstanceDatastore(**Datastores[i]) for i in range(len(Datastores))]
         self.CreatesInContainers = CreatesInContainers
         self.State = State
         self.StateReason = StateReason
         self.CanCreate = CanCreate
         self.LastUpdated = LastUpdated
-
-        # for i in AvailableInstances:
-        #     print(i.keys())
-
         self.AvailableInstances = [Instance(**AvailableInstances[i]) for i in range(len(AvailableInstances))]
         self.AvailableIPs = AvailableIPs
         self.URL = URL
@@ -625,6 +781,8 @@ class LoginResult():
     :type rememberMeToken: str
     :param userInfo: The user information
     :type userInfo: UserInfo
+    :param resultReason: The result reason
+    :type resultReason: str
     :param result: The result
     :type result: float
     """
@@ -633,9 +791,10 @@ class LoginResult():
     sessionID: str
     rememberMeToken: str
     userInfo: UserInfo
+    resultReason: str
     result: float
 
-    def __init__(self, success: bool, permissions: list[str], sessionID: str, rememberMeToken: str, userInfo: UserInfo, result: float) -> None:
+    def __init__(self, success: bool, resultReason: str, result: float, permissions: list[str] = [], sessionID: str = "", rememberMeToken: str = "", userInfo: UserInfo = None) -> None:
         """
         Initializes the LoginResult object
         Author: p0t4t0sandwich
@@ -644,7 +803,11 @@ class LoginResult():
         self.permissions = permissions
         self.sessionID = sessionID
         self.rememberMeToken = rememberMeToken
-        self.userInfo = UserInfo(**userInfo)
+        if userInfo != None:
+            self.userInfo = UserInfo(**userInfo)
+        else:
+            self.userInfo = userInfo
+        self.resultReason = resultReason
         self.result = result
 
 class Message(NamedTuple):
@@ -777,53 +940,6 @@ class RemoteTargetInfo():
         self.Datastores = [InstanceDatastore(**Datastores[i]) for i in range(len(Datastores))]
         self.DeploysInContainers = DeploysInContainers
 
-class Result(Generic[T]):
-    """
-    Generic response type for calls that return a result
-    Author: p0t4t0sandwich
-    :param result: The result object
-    :type result: T
-    """
-    result: T
-
-    def __init__(self, result: T) -> None:
-        """
-        Initializes the Result object
-        Author: p0t4t0sandwich
-        """
-
-        # TODO: Replace this with a more elegant solution
-
-        if type(result) == list:
-            if len(result) == 0:
-                self.result = []
-                return
-
-            keys = result[0].keys()
-            if "DisplayName" in keys and "Endpoint" in keys and "Uri" in keys:
-                self.result = [EndpointInfo(**result[i]) for i in range(len(result))]
-            elif "Id" in keys and "InstanceId" in keys and "FriendlyName" in keys and "Disabled" in keys and "IsRemote" in keys and "Platform" in keys and "Datastores" in keys and "CreatesInContainers" in keys and "State" in keys and "StateReason" in keys and "CanCreate" in keys and "LastUpdated" in keys and "AvailableInstances" in keys and "AvailableIPs" in keys:
-                self.result = [IADSInstance(**result[i]) for i in range(len(result))]
-            elif "IsPrimaryTask" in keys and "StartTime" in keys and "Id" in keys and "Name" in keys and "Description" in keys and "HideFromUI" in keys and "FastDismiss" in keys and "LastUpdatePushed" in keys and "ProgressPercent" in keys and "IsCancellable" in keys and "Origin" in keys and "IsIndeterminate" in keys and "State" in keys and "Status" in keys:
-                self.result = [RunningTask(**result[i]) for i in range(len(result))]
-            else:
-                self.result = result
-
-        elif type(result) == dict:
-            keys = result.keys()
-            if "IsPrimaryTask" in keys and "StartTime" in keys and "Id" in keys and "Name" in keys and "Description" in keys and "HideFromUI" in keys and "FastDismiss" in keys and "LastUpdatePushed" in keys and "ProgressPercent" in keys and "IsCancellable" in keys and "Origin" in keys and "IsIndeterminate" in keys and "State" in keys and "Status" in keys:
-                self.result = RunningTask(**result)
-            elif "Id" in keys and "InstanceId" in keys and "FriendlyName" in keys and "Disabled" in keys and "IsRemote" in keys and "Platform" in keys and "Datastores" in keys and "CreatesInContainers" in keys and "State" in keys and "StateReason" in keys and "CanCreate" in keys and "LastUpdated" in keys and "AvailableInstances" in keys and "AvailableIPs" in keys:
-                self.result = IADSInstance(**result)
-            elif "Name" in keys and "Author" in keys and "AppName" in keys and "SupportsSleep" in keys and "LoadedPlugins" in keys and "AMPVersion" in keys and "AMPBuild" in keys and "ToolsVersion" in keys and "APIVersion" in keys and "VersionCodename" in keys and "Timestamp" in keys and "BuildSpec" in keys and "Branding" in keys and "Analytics" in keys and "FeatureSet" in keys and "InstanceId" in keys and "InstanceName" in keys and "FriendlyName" in keys and "EndpointURI" in keys and "PrimaryEndpoint" in keys and "ModuleName" in keys and "IsRemoteInstance" in keys and "DisplayBaseURI" in keys and "RequiresFullLoad" in keys and "AllowRememberMe" in keys:
-                self.result = ModuleInfo(**result)
-            elif "UpdateAvailable" in keys and "Version" in keys and "Build" in keys and "ReleaseNotesURL" in keys and "ToolsVersion" in keys and "PatchOnly" in keys:
-                self.result = UpdateInfo(**result)
-            else:
-                self.result = result
-        else:
-            self.result = result
-
 class RunningTask(NamedTuple):
     """
     A running task object returned by the Core#GetTasks() method
@@ -872,7 +988,7 @@ class RunningTask(NamedTuple):
     State: int
     Status: str
 
-class Spec(NamedTuple):
+class SettingSpec(NamedTuple):
     """
     A setting specification object
     Author: p0t4t0sandwich
@@ -944,22 +1060,6 @@ class Spec(NamedTuple):
     Meta: str
     RequiresRestart: bool
 
-class SettingsSpec():
-    """
-    Response object for Core.GetSettingsSpec()
-    Author: p0t4t0sandwich
-    :param result: The result
-    :type result: dict[str, Spec]
-    """
-    result: dict[str, Spec]
-
-    def __init__(self, result: dict[str, Spec]) -> None:
-        """
-        Initializes the SettingsSpec object
-        Author: p0t4t0sandwich
-        """
-        self.result = {i : Spec(**result[i]) for i in result}
-
 class Status():
     """
     Struct for the result of API.Core.GetStatus
@@ -984,44 +1084,6 @@ class Status():
         self.Uptime = Uptime
         self.Metrics = {i : Metric(**Metrics[i]) for i in Metrics}
 
-class Task(Generic[T]):
-    """
-    Generic response type for calls that return a result
-    Author: p0t4t0sandwich
-    :param result: The result object
-    :type result: T
-    """
-    result: T
-
-    def __init__(self, result: T) -> None:
-        """
-        Initializes the Task object
-        Author: p0t4t0sandwich
-        """
-        if type(result) == list:
-            if len(result) == 0:
-                self.result = []
-                return
-
-            keys = result[0].keys()
-            if "ID" in keys and "Username" in keys and "EmailAddress" in keys and "IsTwoFactorEnabled" in keys and "Disabled" in keys and "LastLogin" in keys and "GravatarHash" in keys and "IsLDAPUser" in keys:
-                self.result = [UserInfo(**result[i]) for i in range(len(result))]
-            else:
-                self.result = result
-
-        elif type(result) == dict:
-            keys = result.keys()
-            if "IsPrimaryTask" in keys and "StartTime" in keys and "Id" in keys and "Name" in keys and "Description" in keys and "HideFromUI" in keys and "FastDismiss" in keys and "LastUpdatePushed" in keys and "ProgressPercent" in keys and "IsCancellable" in keys and "Origin" in keys and "IsIndeterminate" in keys and "State" in keys and "Status" in keys:
-                self.result = RunningTask(**result)
-            elif "ID" in keys and "Username" in keys and "EmailAddress" in keys and "IsTwoFactorEnabled" in keys and "Disabled" in keys and "LastLogin" in keys and "GravatarHash" in keys and "IsLDAPUser" in keys:
-                self.result = UserInfo(**result)
-            elif "Status" in keys and ("Result" in keys or "Reason" in keys):
-                self.result = ActionResult(**result)
-            else:
-                self.result = result
-        else:
-            self.result = result
-
 class UpdateInfo(NamedTuple):
     """
     A struct to represent the object returned by the ADSModule#GetUpdateInfo() method
